"""
    Agilicus API

    Agilicus is API-first. Modern software is controlled by other software, is open, is available for you to use the way you want, securely, simply.  The OpenAPI Specification in YAML format is available on [www](https://www.agilicus.com/www/api/agilicus-openapi.yaml) for importing to other tools.  A rendered, online viewable and usable version of this specification is available at [api](https://www.agilicus.com/api). You may try the API inline directly in the web page. To do so, first obtain an Authentication Token (the simplest way is to install the Python SDK, and then run `agilicus-cli --issuer https://MYISSUER get-token`). You will need an org-id for most calls (and can obtain from `agilicus-cli --issuer https://MYISSUER list-orgs`). The `MYISSUER` will typically be `auth.MYDOMAIN`, and you will see it as you sign-in to the administrative UI.  This API releases on Bearer-Token authentication. To obtain a valid bearer token you will need to Authenticate to an Issuer with OpenID Connect (a superset of OAUTH2).  Your \"issuer\" will look like https://auth.MYDOMAIN. For example, when you signed-up, if you said \"use my own domain name\" and assigned a CNAME of cloud.example.com, then your issuer would be https://auth.cloud.example.com.  If you selected \"use an Agilicus supplied domain name\", your issuer would look like https://auth.myorg.agilicus.cloud.  For test purposes you can use our [Python SDK](https://pypi.org/project/agilicus/) and run `agilicus-cli --issuer https://auth.MYDOMAIN get-token`.  This API may be used in any language runtime that supports OpenAPI 3.0, or, you may use our [Python SDK](https://pypi.org/project/agilicus/), our [Typescript SDK](https://www.npmjs.com/package/@agilicus/angular), or our [Golang SDK](https://git.agilicus.com/pub/sdk-go).  100% of the activities in our system our API-driven, from our web-admin, through our progressive web applications, to all internals: there is nothing that is not accessible.  For more information, see [developer resources](https://www.agilicus.com/developer).   # noqa: E501

    The version of the OpenAPI document: 2025.02.12
    Contact: dev@agilicus.com
    Generated by: https://openapi-generator.tech
"""


import unittest

import agilicus_api
from agilicus_api.api.certificates_api import CertificatesApi  # noqa: E501


class TestCertificatesApi(unittest.TestCase):
    """CertificatesApi unit test stubs"""

    def setUp(self):
        self.api = CertificatesApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_create_cert(self):
        """Test case for create_cert

        Creates a X509Certificate  # noqa: E501
        """
        pass

    def test_delete_agent_csr(self):
        """Test case for delete_agent_csr

        Delete a CertSigningReq  # noqa: E501
        """
        pass

    def test_delete_cert(self):
        """Test case for delete_cert

        Delete a X509Certificate  # noqa: E501
        """
        pass

    def test_delete_csr(self):
        """Test case for delete_csr

        Delete a CertSigningReq  # noqa: E501
        """
        pass

    def test_get_cert(self):
        """Test case for get_cert

        Get a X509Certificate  # noqa: E501
        """
        pass

    def test_get_csr(self):
        """Test case for get_csr

        Get a CertSigningReq  # noqa: E501
        """
        pass

    def test_list_certs(self):
        """Test case for list_certs

        list certificates  # noqa: E501
        """
        pass

    def test_list_csr(self):
        """Test case for list_csr

        list certificate signing requests  # noqa: E501
        """
        pass

    def test_list_root_certs(self):
        """Test case for list_root_certs

        list root certificates  # noqa: E501
        """
        pass

    def test_reissue_cert_for_csr(self):
        """Test case for reissue_cert_for_csr

        Creates a CertSigningRequestReissue  # noqa: E501
        """
        pass

    def test_replace_csr(self):
        """Test case for replace_csr

        Update a CertSigningReq  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



from unittest.mock import MagicMock

class CredentialsApiMock:

    def __init__(self):
        self.mock_create_object_credential = MagicMock()
        self.mock_delete_object_credential = MagicMock()
        self.mock_get_object_credential = MagicMock()
        self.mock_list_object_credential_existence_info = MagicMock()
        self.mock_list_object_credentials = MagicMock()
        self.mock_replace_object_credential = MagicMock()

    def create_object_credential(self, *args, **kwargs):
        """
        This method mocks the original api CredentialsApi.create_object_credential with MagicMock.
        """
        return self.mock_create_object_credential(self, *args, **kwargs)

    def delete_object_credential(self, *args, **kwargs):
        """
        This method mocks the original api CredentialsApi.delete_object_credential with MagicMock.
        """
        return self.mock_delete_object_credential(self, *args, **kwargs)

    def get_object_credential(self, *args, **kwargs):
        """
        This method mocks the original api CredentialsApi.get_object_credential with MagicMock.
        """
        return self.mock_get_object_credential(self, *args, **kwargs)

    def list_object_credential_existence_info(self, *args, **kwargs):
        """
        This method mocks the original api CredentialsApi.list_object_credential_existence_info with MagicMock.
        """
        return self.mock_list_object_credential_existence_info(self, *args, **kwargs)

    def list_object_credentials(self, *args, **kwargs):
        """
        This method mocks the original api CredentialsApi.list_object_credentials with MagicMock.
        """
        return self.mock_list_object_credentials(self, *args, **kwargs)

    def replace_object_credential(self, *args, **kwargs):
        """
        This method mocks the original api CredentialsApi.replace_object_credential with MagicMock.
        """
        return self.mock_replace_object_credential(self, *args, **kwargs)




# flake8: noqa

"""
    Agilicus API

    Agilicus is API-first. Modern software is controlled by other software, is open, is available for you to use the way you want, securely, simply.  The OpenAPI Specification in YAML format is available on [www](https://www.agilicus.com/www/api/agilicus-openapi.yaml) for importing to other tools.  A rendered, online viewable and usable version of this specification is available at [api](https://www.agilicus.com/api). You may try the API inline directly in the web page. To do so, first obtain an Authentication Token (the simplest way is to install the Python SDK, and then run `agilicus-cli --issuer https://MYISSUER get-token`). You will need an org-id for most calls (and can obtain from `agilicus-cli --issuer https://MYISSUER list-orgs`). The `MYISSUER` will typically be `auth.MYDOMAIN`, and you will see it as you sign-in to the administrative UI.  This API releases on Bearer-Token authentication. To obtain a valid bearer token you will need to Authenticate to an Issuer with OpenID Connect (a superset of OAUTH2).  Your \"issuer\" will look like https://auth.MYDOMAIN. For example, when you signed-up, if you said \"use my own domain name\" and assigned a CNAME of cloud.example.com, then your issuer would be https://auth.cloud.example.com.  If you selected \"use an Agilicus supplied domain name\", your issuer would look like https://auth.myorg.agilicus.cloud.  For test purposes you can use our [Python SDK](https://pypi.org/project/agilicus/) and run `agilicus-cli --issuer https://auth.MYDOMAIN get-token`.  This API may be used in any language runtime that supports OpenAPI 3.0, or, you may use our [Python SDK](https://pypi.org/project/agilicus/), our [Typescript SDK](https://www.npmjs.com/package/@agilicus/angular), or our [Golang SDK](https://git.agilicus.com/pub/sdk-go).  100% of the activities in our system our API-driven, from our web-admin, through our progressive web applications, to all internals: there is nothing that is not accessible.  For more information, see [developer resources](https://www.agilicus.com/developer).   # noqa: E501

    The version of the OpenAPI document: 2025.02.12
    Contact: dev@agilicus.com
    Generated by: https://openapi-generator.tech
"""



# import ApiClient
from agilicus_api.api_client import ApiClient

# import Configuration
from agilicus_api.configuration import Configuration

from agilicus_api.api.application_services_api import ApplicationServicesApi
from agilicus_api.api.applications_api import ApplicationsApi
from agilicus_api.api.audits_api import AuditsApi
from agilicus_api.api.billing_api import BillingApi
from agilicus_api.api.catalogues_api import CataloguesApi
from agilicus_api.api.certificates_api import CertificatesApi
from agilicus_api.api.challenges_api import ChallengesApi
from agilicus_api.api.connectors_api import ConnectorsApi
from agilicus_api.api.credentials_api import CredentialsApi
from agilicus_api.api.diagnostics_api import DiagnosticsApi
from agilicus_api.api.features_api import FeaturesApi
from agilicus_api.api.files_api import FilesApi
from agilicus_api.api.groups_api import GroupsApi
from agilicus_api.api.hosts_api import HostsApi
from agilicus_api.api.issuers_api import IssuersApi
from agilicus_api.api.labels_api import LabelsApi
from agilicus_api.api.launchers_api import LaunchersApi
from agilicus_api.api.lookups_api import LookupsApi
from agilicus_api.api.messages_api import MessagesApi
from agilicus_api.api.metrics_api import MetricsApi
from agilicus_api.api.organisations_api import OrganisationsApi
from agilicus_api.api.permissions_api import PermissionsApi
from agilicus_api.api.policy_api import PolicyApi
from agilicus_api.api.policy_templates_api import PolicyTemplatesApi
from agilicus_api.api.policy_config_api import PolicyConfigApi
from agilicus_api.api.regions_api import RegionsApi
from agilicus_api.api.resources_api import ResourcesApi
from agilicus_api.api.rules_api import RulesApi
from agilicus_api.api.tokens_api import TokensApi
from agilicus_api.api.trusted_certs_api import TrustedCertsApi
from agilicus_api.api.users_api import UsersApi
from agilicus_api.api.whoami_api import WhoamiApi


# import exceptions
from agilicus_api.exceptions import OpenApiException
#from agilicus_api.exceptions import ApiAttributeError
from agilicus_api.exceptions import ApiTypeError
from agilicus_api.exceptions import ApiValueError
from agilicus_api.exceptions import ApiKeyError
from agilicus_api.exceptions import ApiException

from agilicus_api.model.api_key import APIKey
from agilicus_api.model.api_key_introspect import APIKeyIntrospect
from agilicus_api.model.api_key_introspect_authorization_info import APIKeyIntrospectAuthorizationInfo
from agilicus_api.model.api_key_introspect_response import APIKeyIntrospectResponse
from agilicus_api.model.api_key_op_status import APIKeyOpStatus
from agilicus_api.model.api_key_spec import APIKeySpec
from agilicus_api.model.api_key_status import APIKeyStatus
from agilicus_api.model.access_requests import AccessRequests
from agilicus_api.model.access_requests_status import AccessRequestsStatus
from agilicus_api.model.add_group_member_request import AddGroupMemberRequest
from agilicus_api.model.admin_status import AdminStatus
from agilicus_api.model.agent_connector import AgentConnector
from agilicus_api.model.agent_connector_authz_stats import AgentConnectorAuthzStats
from agilicus_api.model.agent_connector_bootstrap import AgentConnectorBootstrap
from agilicus_api.model.agent_connector_cloud_routing import AgentConnectorCloudRouting
from agilicus_api.model.agent_connector_connection_info import AgentConnectorConnectionInfo
from agilicus_api.model.agent_connector_dynamic_stats import AgentConnectorDynamicStats
from agilicus_api.model.agent_connector_dynamic_stats_publish import AgentConnectorDynamicStatsPublish
from agilicus_api.model.agent_connector_info import AgentConnectorInfo
from agilicus_api.model.agent_connector_instance import AgentConnectorInstance
from agilicus_api.model.agent_connector_instance_spec import AgentConnectorInstanceSpec
from agilicus_api.model.agent_connector_instance_status import AgentConnectorInstanceStatus
from agilicus_api.model.agent_connector_local_bind import AgentConnectorLocalBind
from agilicus_api.model.agent_connector_local_bind_error import AgentConnectorLocalBindError
from agilicus_api.model.agent_connector_local_bind_stats import AgentConnectorLocalBindStats
from agilicus_api.model.agent_connector_on_demand_routing_stats import AgentConnectorOnDemandRoutingStats
from agilicus_api.model.agent_connector_outer_proxy_info import AgentConnectorOuterProxyInfo
from agilicus_api.model.agent_connector_per_share_stats import AgentConnectorPerShareStats
from agilicus_api.model.agent_connector_per_user_stats import AgentConnectorPerUserStats
from agilicus_api.model.agent_connector_proxy import AgentConnectorProxy
from agilicus_api.model.agent_connector_proxy_request_stats import AgentConnectorProxyRequestStats
from agilicus_api.model.agent_connector_proxy_request_stats_details import AgentConnectorProxyRequestStatsDetails
from agilicus_api.model.agent_connector_proxy_spec import AgentConnectorProxySpec
from agilicus_api.model.agent_connector_proxy_stats import AgentConnectorProxyStats
from agilicus_api.model.agent_connector_proxy_status import AgentConnectorProxyStatus
from agilicus_api.model.agent_connector_queue import AgentConnectorQueue
from agilicus_api.model.agent_connector_queue_response import AgentConnectorQueueResponse
from agilicus_api.model.agent_connector_queue_spec import AgentConnectorQueueSpec
from agilicus_api.model.agent_connector_queue_status import AgentConnectorQueueStatus
from agilicus_api.model.agent_connector_routing_stats import AgentConnectorRoutingStats
from agilicus_api.model.agent_connector_share_stats import AgentConnectorShareStats
from agilicus_api.model.agent_connector_spec import AgentConnectorSpec
from agilicus_api.model.agent_connector_spec_provisioning import AgentConnectorSpecProvisioning
from agilicus_api.model.agent_connector_specific_stats import AgentConnectorSpecificStats
from agilicus_api.model.agent_connector_stats import AgentConnectorStats
from agilicus_api.model.agent_connector_status import AgentConnectorStatus
from agilicus_api.model.agent_connector_system_stats import AgentConnectorSystemStats
from agilicus_api.model.agent_connector_transport_stats import AgentConnectorTransportStats
from agilicus_api.model.agent_connector_tunnel_info import AgentConnectorTunnelInfo
from agilicus_api.model.agent_connector_tunneling import AgentConnectorTunneling
from agilicus_api.model.agent_connector_user_stats import AgentConnectorUserStats
from agilicus_api.model.agent_local_auth_info import AgentLocalAuthInfo
from agilicus_api.model.allow_map_compiled import AllowMapCompiled
from agilicus_api.model.allow_rule_compiled import AllowRuleCompiled
from agilicus_api.model.alternate_mode_setting import AlternateModeSetting
from agilicus_api.model.alternate_mode_status import AlternateModeStatus
from agilicus_api.model.application import Application
from agilicus_api.model.application_additional_context import ApplicationAdditionalContext
from agilicus_api.model.application_assignment import ApplicationAssignment
from agilicus_api.model.application_authentication_config import ApplicationAuthenticationConfig
from agilicus_api.model.application_config import ApplicationConfig
from agilicus_api.model.application_dynamic_config import ApplicationDynamicConfig
from agilicus_api.model.application_monitoring_config import ApplicationMonitoringConfig
from agilicus_api.model.application_security import ApplicationSecurity
from agilicus_api.model.application_service import ApplicationService
from agilicus_api.model.application_service_assignment import ApplicationServiceAssignment
from agilicus_api.model.application_service_common_stats import ApplicationServiceCommonStats
from agilicus_api.model.application_service_load_balancing import ApplicationServiceLoadBalancing
from agilicus_api.model.application_service_location import ApplicationServiceLocation
from agilicus_api.model.application_service_route import ApplicationServiceRoute
from agilicus_api.model.application_service_routing_info import ApplicationServiceRoutingInfo
from agilicus_api.model.application_service_stats import ApplicationServiceStats
from agilicus_api.model.application_service_stats_group import ApplicationServiceStatsGroup
from agilicus_api.model.application_service_stats_publish import ApplicationServiceStatsPublish
from agilicus_api.model.application_service_status import ApplicationServiceStatus
from agilicus_api.model.application_state_selector import ApplicationStateSelector
from agilicus_api.model.application_stats import ApplicationStats
from agilicus_api.model.application_stats_list import ApplicationStatsList
from agilicus_api.model.application_summary import ApplicationSummary
from agilicus_api.model.application_summary_status import ApplicationSummaryStatus
from agilicus_api.model.application_upstream_config import ApplicationUpstreamConfig
from agilicus_api.model.application_upstream_form_info import ApplicationUpstreamFormInfo
from agilicus_api.model.application_upstream_identity_provider import ApplicationUpstreamIdentityProvider
from agilicus_api.model.application_upstream_validation import ApplicationUpstreamValidation
from agilicus_api.model.applied_connector_stats_config import AppliedConnectorStatsConfig
from agilicus_api.model.audit import Audit
from agilicus_api.model.audit_attribute import AuditAttribute
from agilicus_api.model.audit_destination import AuditDestination
from agilicus_api.model.audit_destination_authentication import AuditDestinationAuthentication
from agilicus_api.model.audit_destination_filter import AuditDestinationFilter
from agilicus_api.model.audit_destination_spec import AuditDestinationSpec
from agilicus_api.model.audit_event import AuditEvent
from agilicus_api.model.audit_event_response import AuditEventResponse
from agilicus_api.model.audit_webhook_bulk_event import AuditWebhookBulkEvent
from agilicus_api.model.audit_webhook_events_processed import AuditWebhookEventsProcessed
from agilicus_api.model.auth_audits import AuthAudits
from agilicus_api.model.authentication_attribute import AuthenticationAttribute
from agilicus_api.model.authentication_document import AuthenticationDocument
from agilicus_api.model.authentication_document_spec import AuthenticationDocumentSpec
from agilicus_api.model.authentication_document_status import AuthenticationDocumentStatus
from agilicus_api.model.authz_bundle import AuthzBundle
from agilicus_api.model.auto_create_status import AutoCreateStatus
from agilicus_api.model.base_upstream import BaseUpstream
from agilicus_api.model.base_upstreams import BaseUpstreams
from agilicus_api.model.billing_account import BillingAccount
from agilicus_api.model.billing_account_spec import BillingAccountSpec
from agilicus_api.model.billing_account_status import BillingAccountStatus
from agilicus_api.model.billing_balance_transaction import BillingBalanceTransaction
from agilicus_api.model.billing_checkout_session import BillingCheckoutSession
from agilicus_api.model.billing_customer import BillingCustomer
from agilicus_api.model.billing_org import BillingOrg
from agilicus_api.model.billing_org_subscription import BillingOrgSubscription
from agilicus_api.model.billing_org_subscription_balance import BillingOrgSubscriptionBalance
from agilicus_api.model.billing_org_subscription_spec import BillingOrgSubscriptionSpec
from agilicus_api.model.billing_org_subscription_status import BillingOrgSubscriptionStatus
from agilicus_api.model.billing_portal_link import BillingPortalLink
from agilicus_api.model.billing_product import BillingProduct
from agilicus_api.model.billing_product_price import BillingProductPrice
from agilicus_api.model.billing_provider_subscription_status import BillingProviderSubscriptionStatus
from agilicus_api.model.billing_subscription import BillingSubscription
from agilicus_api.model.billing_subscription_cancel_detail import BillingSubscriptionCancelDetail
from agilicus_api.model.billing_subscription_new_subscription import BillingSubscriptionNewSubscription
from agilicus_api.model.billing_subscription_usage_override_item import BillingSubscriptionUsageOverrideItem
from agilicus_api.model.billing_usage_record import BillingUsageRecord
from agilicus_api.model.billing_usage_summary import BillingUsageSummary
from agilicus_api.model.bulk_delete_labelled_objects_request import BulkDeleteLabelledObjectsRequest
from agilicus_api.model.bulk_delete_labelled_objects_response import BulkDeleteLabelledObjectsResponse
from agilicus_api.model.bulk_session_operation_response import BulkSessionOperationResponse
from agilicus_api.model.bulk_token_revoke import BulkTokenRevoke
from agilicus_api.model.bulk_token_revoke_response import BulkTokenRevokeResponse
from agilicus_api.model.bulk_user_metadata import BulkUserMetadata
from agilicus_api.model.bulk_user_request_approval import BulkUserRequestApproval
from agilicus_api.model.cors_origin import CORSOrigin
from agilicus_api.model.cors_settings import CORSSettings
from agilicus_api.model.csp_directive import CSPDirective
from agilicus_api.model.csp_settings import CSPSettings
from agilicus_api.model.csr_reason_enum import CSRReasonEnum
from agilicus_api.model.catalogue import Catalogue
from agilicus_api.model.catalogue_entry import CatalogueEntry
from agilicus_api.model.cert_signing_req import CertSigningReq
from agilicus_api.model.cert_signing_req_reissue import CertSigningReqReissue
from agilicus_api.model.cert_signing_req_reissue_spec import CertSigningReqReissueSpec
from agilicus_api.model.cert_signing_req_reissue_status import CertSigningReqReissueStatus
from agilicus_api.model.cert_signing_req_spec import CertSigningReqSpec
from agilicus_api.model.cert_signing_req_status import CertSigningReqStatus
from agilicus_api.model.certificate_revocation_proxy import CertificateRevocationProxy
from agilicus_api.model.certificate_transparency_settings import CertificateTransparencySettings
from agilicus_api.model.challenge import Challenge
from agilicus_api.model.challenge_action import ChallengeAction
from agilicus_api.model.challenge_actor import ChallengeActor
from agilicus_api.model.challenge_answer import ChallengeAnswer
from agilicus_api.model.challenge_answer_spec import ChallengeAnswerSpec
from agilicus_api.model.challenge_endpoint import ChallengeEndpoint
from agilicus_api.model.challenge_spec import ChallengeSpec
from agilicus_api.model.challenge_status import ChallengeStatus
from agilicus_api.model.cipher_diffie_hellman_group import CipherDiffieHellmanGroup
from agilicus_api.model.cipher_encryption_algorithm import CipherEncryptionAlgorithm
from agilicus_api.model.cipher_integrity_algorithm import CipherIntegrityAlgorithm
from agilicus_api.model.clear_file_association_request import ClearFileAssociationRequest
from agilicus_api.model.clear_file_association_response import ClearFileAssociationResponse
from agilicus_api.model.cluster import Cluster
from agilicus_api.model.cluster_config import ClusterConfig
from agilicus_api.model.cluster_spec import ClusterSpec
from agilicus_api.model.combined_resource_rules import CombinedResourceRules
from agilicus_api.model.combined_resource_rules_status import CombinedResourceRulesStatus
from agilicus_api.model.combined_rules import CombinedRules
from agilicus_api.model.combined_rules_status import CombinedRulesStatus
from agilicus_api.model.combined_user_detail import CombinedUserDetail
from agilicus_api.model.combined_user_detail_status import CombinedUserDetailStatus
from agilicus_api.model.common_metadata import CommonMetadata
from agilicus_api.model.compound_rule_condition import CompoundRuleCondition
from agilicus_api.model.config_file_format import ConfigFileFormat
from agilicus_api.model.configure_connector_stats_publishing_request import ConfigureConnectorStatsPublishingRequest
from agilicus_api.model.connector import Connector
from agilicus_api.model.connector_base_info import ConnectorBaseInfo
from agilicus_api.model.connector_base_info_map import ConnectorBaseInfoMap
from agilicus_api.model.connector_cloud_routing import ConnectorCloudRouting
from agilicus_api.model.connector_diagnostic_stats import ConnectorDiagnosticStats
from agilicus_api.model.connector_instance import ConnectorInstance
from agilicus_api.model.connector_instance_spec import ConnectorInstanceSpec
from agilicus_api.model.connector_instance_status import ConnectorInstanceStatus
from agilicus_api.model.connector_route import ConnectorRoute
from agilicus_api.model.connector_secure_transfer import ConnectorSecureTransfer
from agilicus_api.model.connector_secure_transfer_spec import ConnectorSecureTransferSpec
from agilicus_api.model.connector_secure_transfer_status import ConnectorSecureTransferStatus
from agilicus_api.model.connector_service import ConnectorService
from agilicus_api.model.connector_service_spec import ConnectorServiceSpec
from agilicus_api.model.connector_service_status import ConnectorServiceStatus
from agilicus_api.model.connector_spec import ConnectorSpec
from agilicus_api.model.connector_static_stats import ConnectorStaticStats
from agilicus_api.model.connector_stats import ConnectorStats
from agilicus_api.model.connector_stats_metadata import ConnectorStatsMetadata
from agilicus_api.model.connector_status import ConnectorStatus
from agilicus_api.model.connector_system_stats import ConnectorSystemStats
from agilicus_api.model.connector_upstream_stats_publish import ConnectorUpstreamStatsPublish
from agilicus_api.model.content_type_options_settings import ContentTypeOptionsSettings
from agilicus_api.model.create_billing_checkout_session import CreateBillingCheckoutSession
from agilicus_api.model.create_billing_usage_records import CreateBillingUsageRecords
from agilicus_api.model.create_session_and_token_request import CreateSessionAndTokenRequest
from agilicus_api.model.create_session_and_token_response import CreateSessionAndTokenResponse
from agilicus_api.model.create_token_request import CreateTokenRequest
from agilicus_api.model.create_user_data_token_request import CreateUserDataTokenRequest
from agilicus_api.model.credential_purpose import CredentialPurpose
from agilicus_api.model.cross_origin_embedder_policy_settings import CrossOriginEmbedderPolicySettings
from agilicus_api.model.cross_origin_opener_policy_settings import CrossOriginOpenerPolicySettings
from agilicus_api.model.cross_origin_resource_policy_settings import CrossOriginResourcePolicySettings
from agilicus_api.model.custom_desktop_client_config import CustomDesktopClientConfig
from agilicus_api.model.definition import Definition
from agilicus_api.model.dereferenced_standalone_rule_tree_node import DereferencedStandaloneRuleTreeNode
from agilicus_api.model.dereferenced_standalone_rule_tree_node_child import DereferencedStandaloneRuleTreeNodeChild
from agilicus_api.model.desktop_client_config_item import DesktopClientConfigItem
from agilicus_api.model.desktop_client_configuration import DesktopClientConfiguration
from agilicus_api.model.desktop_client_generated_configuration import DesktopClientGeneratedConfiguration
from agilicus_api.model.desktop_connection_info import DesktopConnectionInfo
from agilicus_api.model.desktop_remote_app import DesktopRemoteApp
from agilicus_api.model.desktop_resource import DesktopResource
from agilicus_api.model.desktop_resource_spec import DesktopResourceSpec
from agilicus_api.model.desktop_resource_stats import DesktopResourceStats
from agilicus_api.model.desktop_resource_status import DesktopResourceStatus
from agilicus_api.model.desktop_server_configuration import DesktopServerConfiguration
from agilicus_api.model.desktop_server_generated_configuration import DesktopServerGeneratedConfiguration
from agilicus_api.model.display_info import DisplayInfo
from agilicus_api.model.domain import Domain
from agilicus_api.model.egress_gateway import EgressGateway
from agilicus_api.model.email import Email
from agilicus_api.model.emptiable_object_type import EmptiableObjectType
from agilicus_api.model.encrypted_data import EncryptedData
from agilicus_api.model.environment import Environment
from agilicus_api.model.environment_config import EnvironmentConfig
from agilicus_api.model.environment_config_var import EnvironmentConfigVar
from agilicus_api.model.environment_status import EnvironmentStatus
from agilicus_api.model.error_message import ErrorMessage
from agilicus_api.model.extra_process import ExtraProcess
from agilicus_api.model.feature import Feature
from agilicus_api.model.feature_flag import FeatureFlag
from agilicus_api.model.feature_key import FeatureKey
from agilicus_api.model.feature_spec import FeatureSpec
from agilicus_api.model.feature_status import FeatureStatus
from agilicus_api.model.feature_tag import FeatureTag
from agilicus_api.model.feature_tag_name import FeatureTagName
from agilicus_api.model.feature_tag_spec import FeatureTagSpec
from agilicus_api.model.feature_value import FeatureValue
from agilicus_api.model.file import File
from agilicus_api.model.file_association import FileAssociation
from agilicus_api.model.file_association_spec import FileAssociationSpec
from agilicus_api.model.file_association_status import FileAssociationStatus
from agilicus_api.model.file_name import FileName
from agilicus_api.model.file_share_client_config import FileShareClientConfig
from agilicus_api.model.file_share_client_config_linux_config import FileShareClientConfigLinuxConfig
from agilicus_api.model.file_share_client_config_mac_config import FileShareClientConfigMacConfig
from agilicus_api.model.file_share_client_config_windows_config import FileShareClientConfigWindowsConfig
from agilicus_api.model.file_share_service import FileShareService
from agilicus_api.model.file_share_service_spec import FileShareServiceSpec
from agilicus_api.model.file_share_service_stats import FileShareServiceStats
from agilicus_api.model.file_share_service_stats_group import FileShareServiceStatsGroup
from agilicus_api.model.file_share_service_status import FileShareServiceStatus
from agilicus_api.model.file_summary import FileSummary
from agilicus_api.model.file_visibility import FileVisibility
from agilicus_api.model.frame_options_settings import FrameOptionsSettings
from agilicus_api.model.generic_float_metric import GenericFloatMetric
from agilicus_api.model.generic_int_metric import GenericIntMetric
from agilicus_api.model.group import Group
from agilicus_api.model.group_data import GroupData
from agilicus_api.model.group_member import GroupMember
from agilicus_api.model.group_reconcile_record import GroupReconcileRecord
from agilicus_api.model.guid_metadata import GuidMetadata
from agilicus_api.model.hsts_settings import HSTSSettings
from agilicus_api.model.http_basic_auth import HTTPBasicAuth
from agilicus_api.model.http_bearer_auth import HTTPBearerAuth
from agilicus_api.model.http_challenge_action import HTTPChallengeAction
from agilicus_api.model.http_detailed_stats import HTTPDetailedStats
from agilicus_api.model.http_response_code_counter import HTTPResponseCodeCounter
from agilicus_api.model.http_security_settings import HTTPSecuritySettings
from agilicus_api.model.http_summary_stats import HTTPSummaryStats
from agilicus_api.model.host import Host
from agilicus_api.model.host_bundle import HostBundle
from agilicus_api.model.host_bundle_client_config import HostBundleClientConfig
from agilicus_api.model.host_bundle_label import HostBundleLabel
from agilicus_api.model.host_bundle_name import HostBundleName
from agilicus_api.model.host_bundle_spec import HostBundleSpec
from agilicus_api.model.host_bundle_status import HostBundleStatus
from agilicus_api.model.host_destination import HostDestination
from agilicus_api.model.host_label import HostLabel
from agilicus_api.model.host_label_name import HostLabelName
from agilicus_api.model.host_label_spec import HostLabelSpec
from agilicus_api.model.host_org import HostOrg
from agilicus_api.model.host_permissions import HostPermissions
from agilicus_api.model.host_prefix_rule_condition import HostPrefixRuleCondition
from agilicus_api.model.host_spec import HostSpec
from agilicus_api.model.http_request_extractor import HttpRequestExtractor
from agilicus_api.model.http_request_extractor_source import HttpRequestExtractorSource
from agilicus_api.model.http_rule import HttpRule
from agilicus_api.model.http_rule_condition import HttpRuleCondition
from agilicus_api.model.http_rule_condition_type import HttpRuleConditionType
from agilicus_api.model.icon import Icon
from agilicus_api.model.icon_dimensions import IconDimensions
from agilicus_api.model.icon_purpose import IconPurpose
from agilicus_api.model.identity_assertion import IdentityAssertion
from agilicus_api.model.identity_assertion_response import IdentityAssertionResponse
from agilicus_api.model.inbox_item import InboxItem
from agilicus_api.model.inbox_item_metadata import InboxItemMetadata
from agilicus_api.model.inbox_item_spec import InboxItemSpec
from agilicus_api.model.inbox_item_status import InboxItemStatus
from agilicus_api.model.included_role import IncludedRole
from agilicus_api.model.inheritable_user_config import InheritableUserConfig
from agilicus_api.model.inline_response200 import InlineResponse200
from agilicus_api.model.interceptor_command import InterceptorCommand
from agilicus_api.model.interceptor_config import InterceptorConfig
from agilicus_api.model.invalid_policy_template import InvalidPolicyTemplate
from agilicus_api.model.ipsec_connection import IpsecConnection
from agilicus_api.model.ipsec_connection_ipv4_block import IpsecConnectionIpv4Block
from agilicus_api.model.ipsec_connection_spec import IpsecConnectionSpec
from agilicus_api.model.ipsec_connector import IpsecConnector
from agilicus_api.model.ipsec_connector_spec import IpsecConnectorSpec
from agilicus_api.model.ipsec_connector_status import IpsecConnectorStatus
from agilicus_api.model.ipsec_gateway_interface import IpsecGatewayInterface
from agilicus_api.model.issuer import Issuer
from agilicus_api.model.issuer_client import IssuerClient
from agilicus_api.model.issuer_status import IssuerStatus
from agilicus_api.model.issuer_upstream import IssuerUpstream
from agilicus_api.model.js_inject import JSInject
from agilicus_api.model.json_body_constraint import JSONBodyConstraint
from agilicus_api.model.jwks import JWKS
from agilicus_api.model.k8s_slug import K8sSlug
from agilicus_api.model.kerberos_upstream_config import KerberosUpstreamConfig
from agilicus_api.model.kerberos_upstream_identity_provider import KerberosUpstreamIdentityProvider
from agilicus_api.model.kerberos_upstream_status import KerberosUpstreamStatus
from agilicus_api.model.kerberos_upstream_status_key_tab_entries import KerberosUpstreamStatusKeyTabEntries
from agilicus_api.model.label import Label
from agilicus_api.model.label_association import LabelAssociation
from agilicus_api.model.label_association_status import LabelAssociationStatus
from agilicus_api.model.label_name import LabelName
from agilicus_api.model.label_navigation import LabelNavigation
from agilicus_api.model.label_spec import LabelSpec
from agilicus_api.model.labelled_object import LabelledObject
from agilicus_api.model.latency_bucket_value import LatencyBucketValue
from agilicus_api.model.latency_histogram import LatencyHistogram
from agilicus_api.model.latency_histogram_buckets import LatencyHistogramBuckets
from agilicus_api.model.launcher import Launcher
from agilicus_api.model.launcher_config import LauncherConfig
from agilicus_api.model.launcher_spec import LauncherSpec
from agilicus_api.model.launcher_status import LauncherStatus
from agilicus_api.model.learning_mode_spec import LearningModeSpec
from agilicus_api.model.list_api_keys_response import ListAPIKeysResponse
from agilicus_api.model.list_access_requests_response import ListAccessRequestsResponse
from agilicus_api.model.list_active_users_response import ListActiveUsersResponse
from agilicus_api.model.list_agent_connector_instance_response import ListAgentConnectorInstanceResponse
from agilicus_api.model.list_agent_connector_instance_stats_response import ListAgentConnectorInstanceStatsResponse
from agilicus_api.model.list_agent_connector_proxy_response import ListAgentConnectorProxyResponse
from agilicus_api.model.list_agent_connector_response import ListAgentConnectorResponse
from agilicus_api.model.list_application_services_response import ListApplicationServicesResponse
from agilicus_api.model.list_application_summary_response import ListApplicationSummaryResponse
from agilicus_api.model.list_applications_response import ListApplicationsResponse
from agilicus_api.model.list_audit_destinations_response import ListAuditDestinationsResponse
from agilicus_api.model.list_audits_response import ListAuditsResponse
from agilicus_api.model.list_auth_audits_response import ListAuthAuditsResponse
from agilicus_api.model.list_authentication_document_response import ListAuthenticationDocumentResponse
from agilicus_api.model.list_billing_accounts_response import ListBillingAccountsResponse
from agilicus_api.model.list_billing_balance_transactions import ListBillingBalanceTransactions
from agilicus_api.model.list_billing_checkout_sessions import ListBillingCheckoutSessions
from agilicus_api.model.list_billing_org_subscriptions_response import ListBillingOrgSubscriptionsResponse
from agilicus_api.model.list_billing_usage_records_response import ListBillingUsageRecordsResponse
from agilicus_api.model.list_catalogue_entries_response import ListCatalogueEntriesResponse
from agilicus_api.model.list_catalogues_response import ListCataloguesResponse
from agilicus_api.model.list_cert_signing_req_response import ListCertSigningReqResponse
from agilicus_api.model.list_cluster_response import ListClusterResponse
from agilicus_api.model.list_combined_resource_rules_response import ListCombinedResourceRulesResponse
from agilicus_api.model.list_combined_rules_response import ListCombinedRulesResponse
from agilicus_api.model.list_combined_user_details_response import ListCombinedUserDetailsResponse
from agilicus_api.model.list_configs_response import ListConfigsResponse
from agilicus_api.model.list_connector_response import ListConnectorResponse
from agilicus_api.model.list_connector_secure_transfer_response import ListConnectorSecureTransferResponse
from agilicus_api.model.list_connector_service_response import ListConnectorServiceResponse
from agilicus_api.model.list_connector_stats_response import ListConnectorStatsResponse
from agilicus_api.model.list_desktop_resources_response import ListDesktopResourcesResponse
from agilicus_api.model.list_domains_response import ListDomainsResponse
from agilicus_api.model.list_elevated_user_roles import ListElevatedUserRoles
from agilicus_api.model.list_environment_configs_response import ListEnvironmentConfigsResponse
from agilicus_api.model.list_feature_tags_response import ListFeatureTagsResponse
from agilicus_api.model.list_features_response import ListFeaturesResponse
from agilicus_api.model.list_file_associations_response import ListFileAssociationsResponse
from agilicus_api.model.list_file_share_services_response import ListFileShareServicesResponse
from agilicus_api.model.list_files_response import ListFilesResponse
from agilicus_api.model.list_groups_response import ListGroupsResponse
from agilicus_api.model.list_guid_metadata_response import ListGuidMetadataResponse
from agilicus_api.model.list_host import ListHost
from agilicus_api.model.list_host_bundle import ListHostBundle
from agilicus_api.model.list_host_label import ListHostLabel
from agilicus_api.model.list_host_org import ListHostOrg
from agilicus_api.model.list_inbox_items_response import ListInboxItemsResponse
from agilicus_api.model.list_inbox_items_summary import ListInboxItemsSummary
from agilicus_api.model.list_ipsec_connector_response import ListIpsecConnectorResponse
from agilicus_api.model.list_issuer_clients_response import ListIssuerClientsResponse
from agilicus_api.model.list_issuer_extensions_response import ListIssuerExtensionsResponse
from agilicus_api.model.list_issuer_roots_response import ListIssuerRootsResponse
from agilicus_api.model.list_issuer_upstreams import ListIssuerUpstreams
from agilicus_api.model.list_jwks import ListJWKS
from agilicus_api.model.list_labelled_objects_response import ListLabelledObjectsResponse
from agilicus_api.model.list_labels_response import ListLabelsResponse
from agilicus_api.model.list_launchers_response import ListLaunchersResponse
from agilicus_api.model.list_logs_response import ListLogsResponse
from agilicus_api.model.list_mfa_challenge_methods import ListMFAChallengeMethods
from agilicus_api.model.list_message_endpoints_response import ListMessageEndpointsResponse
from agilicus_api.model.list_object_credential_existence_info_response import ListObjectCredentialExistenceInfoResponse
from agilicus_api.model.list_object_credentials_response import ListObjectCredentialsResponse
from agilicus_api.model.list_orgs_response import ListOrgsResponse
from agilicus_api.model.list_point_of_presences_response import ListPointOfPresencesResponse
from agilicus_api.model.list_policies_response import ListPoliciesResponse
from agilicus_api.model.list_policy_rules_response import ListPolicyRulesResponse
from agilicus_api.model.list_policy_template_instances_response import ListPolicyTemplateInstancesResponse
from agilicus_api.model.list_products_response import ListProductsResponse
from agilicus_api.model.list_public_file_org_links_response import ListPublicFileOrgLinksResponse
from agilicus_api.model.list_resource_groups_response import ListResourceGroupsResponse
from agilicus_api.model.list_resource_permissions_response import ListResourcePermissionsResponse
from agilicus_api.model.list_resource_roles_response import ListResourceRolesResponse
from agilicus_api.model.list_resources_response import ListResourcesResponse
from agilicus_api.model.list_role_to_rule_entries import ListRoleToRuleEntries
from agilicus_api.model.list_roles import ListRoles
from agilicus_api.model.list_rules import ListRules
from agilicus_api.model.list_ssh_resources_response import ListSSHResourcesResponse
from agilicus_api.model.list_secure_agent_response import ListSecureAgentResponse
from agilicus_api.model.list_service_account_response import ListServiceAccountResponse
from agilicus_api.model.list_service_forwarders_response import ListServiceForwardersResponse
from agilicus_api.model.list_sessions_response import ListSessionsResponse
from agilicus_api.model.list_standalone_rule_policies_response import ListStandaloneRulePoliciesResponse
from agilicus_api.model.list_standalone_rule_trees_response import ListStandaloneRuleTreesResponse
from agilicus_api.model.list_standalone_rules_response import ListStandaloneRulesResponse
from agilicus_api.model.list_standalone_ruleset_bundles_response import ListStandaloneRulesetBundlesResponse
from agilicus_api.model.list_standalone_ruleset_labels_response import ListStandaloneRulesetLabelsResponse
from agilicus_api.model.list_standalone_rulesets_response import ListStandaloneRulesetsResponse
from agilicus_api.model.list_support_user_response import ListSupportUserResponse
from agilicus_api.model.list_totp_enrollment_response import ListTOTPEnrollmentResponse
from agilicus_api.model.list_tokens_response import ListTokensResponse
from agilicus_api.model.list_top_users_response import ListTopUsersResponse
from agilicus_api.model.list_trusted_certificate import ListTrustedCertificate
from agilicus_api.model.list_trusted_certificate_bundle import ListTrustedCertificateBundle
from agilicus_api.model.list_trusted_certificate_label import ListTrustedCertificateLabel
from agilicus_api.model.list_trusted_certificate_org import ListTrustedCertificateOrg
from agilicus_api.model.list_upstream_aliases import ListUpstreamAliases
from agilicus_api.model.list_upstream_group_mapping import ListUpstreamGroupMapping
from agilicus_api.model.list_upstream_user_identities_response import ListUpstreamUserIdentitiesResponse
from agilicus_api.model.list_user_application_access_info_response import ListUserApplicationAccessInfoResponse
from agilicus_api.model.list_user_desktop_access_info_response import ListUserDesktopAccessInfoResponse
from agilicus_api.model.list_user_file_share_access_info_response import ListUserFileShareAccessInfoResponse
from agilicus_api.model.list_user_guids_response import ListUserGuidsResponse
from agilicus_api.model.list_user_launcher_access_info_response import ListUserLauncherAccessInfoResponse
from agilicus_api.model.list_user_metadata_response import ListUserMetadataResponse
from agilicus_api.model.list_user_request_info_response import ListUserRequestInfoResponse
from agilicus_api.model.list_user_resource_access_info_response import ListUserResourceAccessInfoResponse
from agilicus_api.model.list_user_roles_for_an_org import ListUserRolesForAnOrg
from agilicus_api.model.list_user_ssh_access_info_response import ListUserSSHAccessInfoResponse
from agilicus_api.model.list_users_response import ListUsersResponse
from agilicus_api.model.list_web_auth_n_enrollment_response import ListWebAuthNEnrollmentResponse
from agilicus_api.model.list_well_known_issuer_info import ListWellKnownIssuerInfo
from agilicus_api.model.list_x509_certificate_response import ListX509CertificateResponse
from agilicus_api.model.local_auth_upstream_config import LocalAuthUpstreamConfig
from agilicus_api.model.local_auth_upstream_identity_provider import LocalAuthUpstreamIdentityProvider
from agilicus_api.model.log import Log
from agilicus_api.model.login_session import LoginSession
from agilicus_api.model.lookup_request import LookupRequest
from agilicus_api.model.mfa_challenge_answer import MFAChallengeAnswer
from agilicus_api.model.mfa_challenge_answer_result import MFAChallengeAnswerResult
from agilicus_api.model.mfa_challenge_method import MFAChallengeMethod
from agilicus_api.model.mfa_challenge_method_spec import MFAChallengeMethodSpec
from agilicus_api.model.mfa_challenge_question import MFAChallengeQuestion
from agilicus_api.model.mfa_challenge_question_input import MFAChallengeQuestionInput
from agilicus_api.model.mfa_challenge_question_login_info import MFAChallengeQuestionLoginInfo
from agilicus_api.model.mfa_enrollment_answer import MFAEnrollmentAnswer
from agilicus_api.model.mfa_enrollment_answer_result import MFAEnrollmentAnswerResult
from agilicus_api.model.mfa_enrollment_question import MFAEnrollmentQuestion
from agilicus_api.model.mfa_enrollment_question_input import MFAEnrollmentQuestionInput
from agilicus_api.model.mfa_enrollment_question_login_info import MFAEnrollmentQuestionLoginInfo
from agilicus_api.model.mfa_policy_template import MFAPolicyTemplate
from agilicus_api.model.managed_upstream_identity_provider import ManagedUpstreamIdentityProvider
from agilicus_api.model.many_org_token_introspect_response import ManyOrgTokenIntrospectResponse
from agilicus_api.model.map_attributes_answer import MapAttributesAnswer
from agilicus_api.model.map_attributes_answer_result import MapAttributesAnswerResult
from agilicus_api.model.map_attributes_question import MapAttributesQuestion
from agilicus_api.model.map_attributes_question_input import MapAttributesQuestionInput
from agilicus_api.model.map_attributes_question_login_info import MapAttributesQuestionLoginInfo
from agilicus_api.model.mapped_attributes import MappedAttributes
from agilicus_api.model.max_mfa_proof_rule_condition import MaxMFAProofRuleCondition
from agilicus_api.model.message import Message
from agilicus_api.model.message_action import MessageAction
from agilicus_api.model.message_address import MessageAddress
from agilicus_api.model.message_class import MessageClass
from agilicus_api.model.message_endpoint import MessageEndpoint
from agilicus_api.model.message_endpoint_metadata import MessageEndpointMetadata
from agilicus_api.model.message_endpoint_spec import MessageEndpointSpec
from agilicus_api.model.message_endpoint_type import MessageEndpointType
from agilicus_api.model.message_endpoint_type_web_push import MessageEndpointTypeWebPush
from agilicus_api.model.message_endpoints_config import MessageEndpointsConfig
from agilicus_api.model.message_send_item import MessageSendItem
from agilicus_api.model.message_send_request import MessageSendRequest
from agilicus_api.model.message_send_response import MessageSendResponse
from agilicus_api.model.message_send_response_item import MessageSendResponseItem
from agilicus_api.model.message_tag import MessageTag
from agilicus_api.model.message_tag_name import MessageTagName
from agilicus_api.model.message_type import MessageType
from agilicus_api.model.messages_bulk_delete_request import MessagesBulkDeleteRequest
from agilicus_api.model.messages_bulk_delete_response import MessagesBulkDeleteResponse
from agilicus_api.model.metadata_with_id import MetadataWithId
from agilicus_api.model.metadata_with_id_all_of import MetadataWithIdAllOf
from agilicus_api.model.metadata_with_only_id import MetadataWithOnlyId
from agilicus_api.model.network_detailed_stats import NetworkDetailedStats
from agilicus_api.model.network_mount_rule_config import NetworkMountRuleConfig
from agilicus_api.model.network_port import NetworkPort
from agilicus_api.model.network_port_range import NetworkPortRange
from agilicus_api.model.network_protocol_condition import NetworkProtocolCondition
from agilicus_api.model.network_service_config import NetworkServiceConfig
from agilicus_api.model.network_summary_stats import NetworkSummaryStats
from agilicus_api.model.next_page_email import NextPageEmail
from agilicus_api.model.oidc_auth_config import OIDCAuthConfig
from agilicus_api.model.oidc_auth_path_config import OIDCAuthPathConfig
from agilicus_api.model.oidc_auth_uri import OIDCAuthURI
from agilicus_api.model.oidc_content_type import OIDCContentType
from agilicus_api.model.oidc_proxy_config import OIDCProxyConfig
from agilicus_api.model.oidc_proxy_content_manipulation import OIDCProxyContentManipulation
from agilicus_api.model.oidc_proxy_domain_mapping import OIDCProxyDomainMapping
from agilicus_api.model.oidc_proxy_domain_name_mapping import OIDCProxyDomainNameMapping
from agilicus_api.model.oidc_proxy_domain_substitution import OIDCProxyDomainSubstitution
from agilicus_api.model.oidc_proxy_header import OIDCProxyHeader
from agilicus_api.model.oidc_proxy_header_mapping import OIDCProxyHeaderMapping
from agilicus_api.model.oidc_proxy_header_match import OIDCProxyHeaderMatch
from agilicus_api.model.oidc_proxy_header_name import OIDCProxyHeaderName
from agilicus_api.model.oidc_proxy_header_override import OIDCProxyHeaderOverride
from agilicus_api.model.oidc_proxy_header_replace import OIDCProxyHeaderReplace
from agilicus_api.model.oidc_proxy_header_rewrite_filter import OIDCProxyHeaderRewriteFilter
from agilicus_api.model.oidc_proxy_header_user_config import OIDCProxyHeaderUserConfig
from agilicus_api.model.oidc_proxy_scope import OIDCProxyScope
from agilicus_api.model.oidc_proxy_standard_header import OIDCProxyStandardHeader
from agilicus_api.model.oidc_proxy_upstream_authentication import OIDCProxyUpstreamAuthentication
from agilicus_api.model.oidc_proxy_upstream_config import OIDCProxyUpstreamConfig
from agilicus_api.model.oidc_proxy_upstream_ntlm import OIDCProxyUpstreamNTLM
from agilicus_api.model.oidc_upstream_identity_provider import OIDCUpstreamIdentityProvider
from agilicus_api.model.object_credential import ObjectCredential
from agilicus_api.model.object_credential_existence_info import ObjectCredentialExistenceInfo
from agilicus_api.model.object_credential_secrets import ObjectCredentialSecrets
from agilicus_api.model.object_credential_spec import ObjectCredentialSpec
from agilicus_api.model.object_credential_status import ObjectCredentialStatus
from agilicus_api.model.object_oper_status import ObjectOperStatus
from agilicus_api.model.object_type import ObjectType
from agilicus_api.model.one_time_use_action_challenge import OneTimeUseActionChallenge
from agilicus_api.model.one_time_use_action_challenge_answer_detail import OneTimeUseActionChallengeAnswerDetail
from agilicus_api.model.one_time_use_action_challenge_response import OneTimeUseActionChallengeResponse
from agilicus_api.model.one_time_use_action_challenge_spec import OneTimeUseActionChallengeSpec
from agilicus_api.model.operational_status import OperationalStatus
from agilicus_api.model.org_info import OrgInfo
from agilicus_api.model.org_scope_patch_document import OrgScopePatchDocument
from agilicus_api.model.organisation import Organisation
from agilicus_api.model.organisation_admin import OrganisationAdmin
from agilicus_api.model.organisation_capabilities import OrganisationCapabilities
from agilicus_api.model.organisation_owner_config import OrganisationOwnerConfig
from agilicus_api.model.organisation_state_selector import OrganisationStateSelector
from agilicus_api.model.organisation_state_status import OrganisationStateStatus
from agilicus_api.model.organisation_status import OrganisationStatus
from agilicus_api.model.organisation_system_options import OrganisationSystemOptions
from agilicus_api.model.per_instance_upstream_stats import PerInstanceUpstreamStats
from agilicus_api.model.per_tunnel_stats import PerTunnelStats
from agilicus_api.model.permitted_cross_domain_policies_settings import PermittedCrossDomainPoliciesSettings
from agilicus_api.model.point_of_presence import PointOfPresence
from agilicus_api.model.point_of_presence_routing import PointOfPresenceRouting
from agilicus_api.model.point_of_presence_spec import PointOfPresenceSpec
from agilicus_api.model.point_of_presence_status import PointOfPresenceStatus
from agilicus_api.model.policy import Policy
from agilicus_api.model.policy_condition import PolicyCondition
from agilicus_api.model.policy_config_authz_bundle import PolicyConfigAuthzBundle
from agilicus_api.model.policy_config_resource_url_bundle import PolicyConfigResourceURLBundle
from agilicus_api.model.policy_group import PolicyGroup
from agilicus_api.model.policy_group_spec import PolicyGroupSpec
from agilicus_api.model.policy_issuer_ref import PolicyIssuerRef
from agilicus_api.model.policy_rule import PolicyRule
from agilicus_api.model.policy_rule_spec import PolicyRuleSpec
from agilicus_api.model.policy_spec import PolicySpec
from agilicus_api.model.policy_status import PolicyStatus
from agilicus_api.model.policy_template import PolicyTemplate
from agilicus_api.model.policy_template_instance import PolicyTemplateInstance
from agilicus_api.model.policy_template_instance_spec import PolicyTemplateInstanceSpec
from agilicus_api.model.previous_page_email import PreviousPageEmail
from agilicus_api.model.product import Product
from agilicus_api.model.product_spec import ProductSpec
from agilicus_api.model.product_status import ProductStatus
from agilicus_api.model.public_file_org_link import PublicFileOrgLink
from agilicus_api.model.public_file_org_link_spec import PublicFileOrgLinkSpec
from agilicus_api.model.raw_token import RawToken
from agilicus_api.model.reconcile_org_default_policy_request import ReconcileOrgDefaultPolicyRequest
from agilicus_api.model.reconcile_org_default_policy_response import ReconcileOrgDefaultPolicyResponse
from agilicus_api.model.reconcile_org_default_policy_response_per_org import ReconcileOrgDefaultPolicyResponsePerOrg
from agilicus_api.model.reconcile_sub_org_issuer_request import ReconcileSubOrgIssuerRequest
from agilicus_api.model.referrer_policy_settings import ReferrerPolicySettings
from agilicus_api.model.refresh_token_request import RefreshTokenRequest
from agilicus_api.model.remote_app_access_info import RemoteAppAccessInfo
from agilicus_api.model.rendered_query_parameter import RenderedQueryParameter
from agilicus_api.model.rendered_resource_permissions import RenderedResourcePermissions
from agilicus_api.model.rendered_rule import RenderedRule
from agilicus_api.model.rendered_rule_body import RenderedRuleBody
from agilicus_api.model.replace_user_role_request import ReplaceUserRoleRequest
from agilicus_api.model.reset_mfa_challenge_method import ResetMFAChallengeMethod
from agilicus_api.model.reset_policy_request import ResetPolicyRequest
from agilicus_api.model.reset_user_identity_request import ResetUserIdentityRequest
from agilicus_api.model.resource import Resource
from agilicus_api.model.resource_config import ResourceConfig
from agilicus_api.model.resource_group import ResourceGroup
from agilicus_api.model.resource_info import ResourceInfo
from agilicus_api.model.resource_member import ResourceMember
from agilicus_api.model.resource_permission import ResourcePermission
from agilicus_api.model.resource_permission_spec import ResourcePermissionSpec
from agilicus_api.model.resource_role import ResourceRole
from agilicus_api.model.resource_role_spec import ResourceRoleSpec
from agilicus_api.model.resource_rule_group import ResourceRuleGroup
from agilicus_api.model.resource_session_stats import ResourceSessionStats
from agilicus_api.model.resource_spec import ResourceSpec
from agilicus_api.model.resource_stats import ResourceStats
from agilicus_api.model.resource_stats_metadata import ResourceStatsMetadata
from agilicus_api.model.resource_status import ResourceStatus
from agilicus_api.model.resource_type_enum import ResourceTypeEnum
from agilicus_api.model.resource_url import ResourceURL
from agilicus_api.model.resource_url_host_info import ResourceURLHostInfo
from agilicus_api.model.resource_url_host_info_map import ResourceURLHostInfoMap
from agilicus_api.model.role import Role
from agilicus_api.model.role_config import RoleConfig
from agilicus_api.model.role_list import RoleList
from agilicus_api.model.role_spec import RoleSpec
from agilicus_api.model.role_to_rule_entry import RoleToRuleEntry
from agilicus_api.model.role_to_rule_entry_spec import RoleToRuleEntrySpec
from agilicus_api.model.role_v2 import RoleV2
from agilicus_api.model.roles import Roles
from agilicus_api.model.roles_config import RolesConfig
from agilicus_api.model.rule import Rule
from agilicus_api.model.rule_action import RuleAction
from agilicus_api.model.rule_condition import RuleCondition
from agilicus_api.model.rule_condition_base import RuleConditionBase
from agilicus_api.model.rule_config import RuleConfig
from agilicus_api.model.rule_match_criteria import RuleMatchCriteria
from agilicus_api.model.rule_matcher import RuleMatcher
from agilicus_api.model.rule_matcher_list import RuleMatcherList
from agilicus_api.model.rule_query_body import RuleQueryBody
from agilicus_api.model.rule_query_body_json import RuleQueryBodyJSON
from agilicus_api.model.rule_query_parameter import RuleQueryParameter
from agilicus_api.model.rule_scope_enum import RuleScopeEnum
from agilicus_api.model.rule_set import RuleSet
from agilicus_api.model.rule_set_component import RuleSetComponent
from agilicus_api.model.rule_set_node import RuleSetNode
from agilicus_api.model.rule_spec import RuleSpec
from agilicus_api.model.rule_v2 import RuleV2
from agilicus_api.model.rules_config import RulesConfig
from agilicus_api.model.runtime_status import RuntimeStatus
from agilicus_api.model.ssh_resource import SSHResource
from agilicus_api.model.ssh_resource_spec import SSHResourceSpec
from agilicus_api.model.ssh_resource_stats import SSHResourceStats
from agilicus_api.model.ssh_resource_status import SSHResourceStatus
from agilicus_api.model.ssh_username import SSHUsername
from agilicus_api.model.scope_condition import ScopeCondition
from agilicus_api.model.secure_agent import SecureAgent
from agilicus_api.model.secure_agent_connector import SecureAgentConnector
from agilicus_api.model.secure_agent_connector_info import SecureAgentConnectorInfo
from agilicus_api.model.secure_agent_spec import SecureAgentSpec
from agilicus_api.model.secure_agent_status import SecureAgentStatus
from agilicus_api.model.selector_tag import SelectorTag
from agilicus_api.model.service_account import ServiceAccount
from agilicus_api.model.service_account_reset_body import ServiceAccountResetBody
from agilicus_api.model.service_account_spec import ServiceAccountSpec
from agilicus_api.model.service_account_status import ServiceAccountStatus
from agilicus_api.model.service_expose_config import ServiceExposeConfig
from agilicus_api.model.service_forwarder import ServiceForwarder
from agilicus_api.model.service_forwarder_spec import ServiceForwarderSpec
from agilicus_api.model.service_forwarder_stats import ServiceForwarderStats
from agilicus_api.model.service_forwarder_stats_group import ServiceForwarderStatsGroup
from agilicus_api.model.service_forwarder_status import ServiceForwarderStatus
from agilicus_api.model.service_http_config import ServiceHttpConfig
from agilicus_api.model.service_protocol_config import ServiceProtocolConfig
from agilicus_api.model.session import Session
from agilicus_api.model.session_challenge import SessionChallenge
from agilicus_api.model.session_challenge_message import SessionChallengeMessage
from agilicus_api.model.session_challenge_spec import SessionChallengeSpec
from agilicus_api.model.session_challenge_status import SessionChallengeStatus
from agilicus_api.model.session_status import SessionStatus
from agilicus_api.model.sessions_spec import SessionsSpec
from agilicus_api.model.share_detailed_stats import ShareDetailedStats
from agilicus_api.model.share_summary_stats import ShareSummaryStats
from agilicus_api.model.simple_resource_policy_template import SimpleResourcePolicyTemplate
from agilicus_api.model.simple_resource_policy_template_structure import SimpleResourcePolicyTemplateStructure
from agilicus_api.model.simple_resource_policy_template_structure_node import SimpleResourcePolicyTemplateStructureNode
from agilicus_api.model.source_iso_country_code_condition import SourceISOCountryCodeCondition
from agilicus_api.model.source_info_policy_template import SourceInfoPolicyTemplate
from agilicus_api.model.source_subnet_condition import SourceSubnetCondition
from agilicus_api.model.standalone_object_conditions import StandaloneObjectConditions
from agilicus_api.model.standalone_rule import StandaloneRule
from agilicus_api.model.standalone_rule_name import StandaloneRuleName
from agilicus_api.model.standalone_rule_policy import StandaloneRulePolicy
from agilicus_api.model.standalone_rule_policy_description import StandaloneRulePolicyDescription
from agilicus_api.model.standalone_rule_policy_spec import StandaloneRulePolicySpec
from agilicus_api.model.standalone_rule_scope import StandaloneRuleScope
from agilicus_api.model.standalone_rule_spec import StandaloneRuleSpec
from agilicus_api.model.standalone_rule_tree import StandaloneRuleTree
from agilicus_api.model.standalone_rule_tree_node import StandaloneRuleTreeNode
from agilicus_api.model.standalone_rule_tree_node_child import StandaloneRuleTreeNodeChild
from agilicus_api.model.standalone_rule_tree_ref import StandaloneRuleTreeRef
from agilicus_api.model.standalone_rule_tree_rule_ref import StandaloneRuleTreeRuleRef
from agilicus_api.model.standalone_rule_tree_spec import StandaloneRuleTreeSpec
from agilicus_api.model.standalone_rule_tree_status import StandaloneRuleTreeStatus
from agilicus_api.model.standalone_rules_cleanup import StandaloneRulesCleanup
from agilicus_api.model.standalone_rules_cleanup_spec import StandaloneRulesCleanupSpec
from agilicus_api.model.standalone_rules_cleanup_status import StandaloneRulesCleanupStatus
from agilicus_api.model.standalone_ruleset import StandaloneRuleset
from agilicus_api.model.standalone_ruleset_bundle import StandaloneRulesetBundle
from agilicus_api.model.standalone_ruleset_bundle_label import StandaloneRulesetBundleLabel
from agilicus_api.model.standalone_ruleset_bundle_name import StandaloneRulesetBundleName
from agilicus_api.model.standalone_ruleset_bundle_spec import StandaloneRulesetBundleSpec
from agilicus_api.model.standalone_ruleset_bundle_status import StandaloneRulesetBundleStatus
from agilicus_api.model.standalone_ruleset_in_bundle import StandaloneRulesetInBundle
from agilicus_api.model.standalone_ruleset_label import StandaloneRulesetLabel
from agilicus_api.model.standalone_ruleset_label_name import StandaloneRulesetLabelName
from agilicus_api.model.standalone_ruleset_label_spec import StandaloneRulesetLabelSpec
from agilicus_api.model.standalone_ruleset_label_status import StandaloneRulesetLabelStatus
from agilicus_api.model.standalone_ruleset_spec import StandaloneRulesetSpec
from agilicus_api.model.standalone_ruleset_status import StandaloneRulesetStatus
from agilicus_api.model.stats_publishing_config import StatsPublishingConfig
from agilicus_api.model.stats_publishing_connector_config import StatsPublishingConnectorConfig
from agilicus_api.model.stats_publishing_connector_type_config import StatsPublishingConnectorTypeConfig
from agilicus_api.model.stats_publishing_level_config import StatsPublishingLevelConfig
from agilicus_api.model.storage_region import StorageRegion
from agilicus_api.model.support_user import SupportUser
from agilicus_api.model.support_user_spec import SupportUserSpec
from agilicus_api.model.support_user_status import SupportUserStatus
from agilicus_api.model.totp_enrollment import TOTPEnrollment
from agilicus_api.model.totp_enrollment_answer import TOTPEnrollmentAnswer
from agilicus_api.model.totp_enrollment_spec import TOTPEnrollmentSpec
from agilicus_api.model.totp_enrollment_status import TOTPEnrollmentStatus
from agilicus_api.model.template_extraction import TemplateExtraction
from agilicus_api.model.template_path import TemplatePath
from agilicus_api.model.time_interval_metrics import TimeIntervalMetrics
from agilicus_api.model.time_validity import TimeValidity
from agilicus_api.model.token import Token
from agilicus_api.model.token_introspect import TokenIntrospect
from agilicus_api.model.token_introspect_options import TokenIntrospectOptions
from agilicus_api.model.token_reissue_request import TokenReissueRequest
from agilicus_api.model.token_revoke import TokenRevoke
from agilicus_api.model.token_scope import TokenScope
from agilicus_api.model.token_validity import TokenValidity
from agilicus_api.model.trusted_certificate import TrustedCertificate
from agilicus_api.model.trusted_certificate_bundle import TrustedCertificateBundle
from agilicus_api.model.trusted_certificate_bundle_label import TrustedCertificateBundleLabel
from agilicus_api.model.trusted_certificate_bundle_label_operation import TrustedCertificateBundleLabelOperation
from agilicus_api.model.trusted_certificate_bundle_name import TrustedCertificateBundleName
from agilicus_api.model.trusted_certificate_bundle_spec import TrustedCertificateBundleSpec
from agilicus_api.model.trusted_certificate_bundle_status import TrustedCertificateBundleStatus
from agilicus_api.model.trusted_certificate_label import TrustedCertificateLabel
from agilicus_api.model.trusted_certificate_label_name import TrustedCertificateLabelName
from agilicus_api.model.trusted_certificate_label_spec import TrustedCertificateLabelSpec
from agilicus_api.model.trusted_certificate_label_status import TrustedCertificateLabelStatus
from agilicus_api.model.trusted_certificate_org import TrustedCertificateOrg
from agilicus_api.model.trusted_certificate_spec import TrustedCertificateSpec
from agilicus_api.model.trusted_certificate_status import TrustedCertificateStatus
from agilicus_api.model.uri_parameter_rewrite_filter import URIParameterRewriteFilter
from agilicus_api.model.upstream_alias import UpstreamAlias
from agilicus_api.model.upstream_alias_mapping import UpstreamAliasMapping
from agilicus_api.model.upstream_alias_spec import UpstreamAliasSpec
from agilicus_api.model.upstream_group_excluded_entry import UpstreamGroupExcludedEntry
from agilicus_api.model.upstream_group_mapping import UpstreamGroupMapping
from agilicus_api.model.upstream_group_mapping_entry import UpstreamGroupMappingEntry
from agilicus_api.model.upstream_group_mapping_spec import UpstreamGroupMappingSpec
from agilicus_api.model.upstream_group_reconcile import UpstreamGroupReconcile
from agilicus_api.model.upstream_group_reconcile_response import UpstreamGroupReconcileResponse
from agilicus_api.model.upstream_user_identity import UpstreamUserIdentity
from agilicus_api.model.upstream_user_identity_spec import UpstreamUserIdentitySpec
from agilicus_api.model.usage_measurement import UsageMeasurement
from agilicus_api.model.usage_metric import UsageMetric
from agilicus_api.model.usage_metrics import UsageMetrics
from agilicus_api.model.user import User
from agilicus_api.model.user_application_access_info import UserApplicationAccessInfo
from agilicus_api.model.user_application_access_info_status import UserApplicationAccessInfoStatus
from agilicus_api.model.user_attribute import UserAttribute
from agilicus_api.model.user_attributes import UserAttributes
from agilicus_api.model.user_desktop_access_info import UserDesktopAccessInfo
from agilicus_api.model.user_desktop_access_info_status import UserDesktopAccessInfoStatus
from agilicus_api.model.user_file_share_access_info import UserFileShareAccessInfo
from agilicus_api.model.user_file_share_access_info_status import UserFileShareAccessInfoStatus
from agilicus_api.model.user_identity import UserIdentity
from agilicus_api.model.user_identity_update import UserIdentityUpdate
from agilicus_api.model.user_identity_update_spec import UserIdentityUpdateSpec
from agilicus_api.model.user_info import UserInfo
from agilicus_api.model.user_launcher_access_info import UserLauncherAccessInfo
from agilicus_api.model.user_member_of import UserMemberOf
from agilicus_api.model.user_metadata import UserMetadata
from agilicus_api.model.user_metadata_spec import UserMetadataSpec
from agilicus_api.model.user_metrics import UserMetrics
from agilicus_api.model.user_request_info import UserRequestInfo
from agilicus_api.model.user_request_info_spec import UserRequestInfoSpec
from agilicus_api.model.user_request_info_status import UserRequestInfoStatus
from agilicus_api.model.user_request_user_update import UserRequestUserUpdate
from agilicus_api.model.user_resource_access_info import UserResourceAccessInfo
from agilicus_api.model.user_resource_access_info_status import UserResourceAccessInfoStatus
from agilicus_api.model.user_roles import UserRoles
from agilicus_api.model.user_roles_for_an_org import UserRolesForAnOrg
from agilicus_api.model.user_ssh_access_info import UserSSHAccessInfo
from agilicus_api.model.user_ssh_access_info_status import UserSSHAccessInfoStatus
from agilicus_api.model.user_session_identifiers import UserSessionIdentifiers
from agilicus_api.model.user_status_enum import UserStatusEnum
from agilicus_api.model.user_summary import UserSummary
from agilicus_api.model.vnc_connection_info import VNCConnectionInfo
from agilicus_api.model.vnc_password_authentication import VNCPasswordAuthentication
from agilicus_api.model.web_auth_n_enrollment import WebAuthNEnrollment
from agilicus_api.model.web_auth_n_enrollment_answer import WebAuthNEnrollmentAnswer
from agilicus_api.model.web_auth_n_enrollment_spec import WebAuthNEnrollmentSpec
from agilicus_api.model.web_auth_n_enrollment_status import WebAuthNEnrollmentStatus
from agilicus_api.model.well_known_issuer_info import WellKnownIssuerInfo
from agilicus_api.model.whoami_request import WhoamiRequest
from agilicus_api.model.whoami_response import WhoamiResponse
from agilicus_api.model.workload_configuration import WorkloadConfiguration
from agilicus_api.model.x509_certificate import X509Certificate
from agilicus_api.model.x509_certificate_spec import X509CertificateSpec
from agilicus_api.model.x509_certificate_status import X509CertificateStatus
from agilicus_api.model.x509_root_certificate import X509RootCertificate
from agilicus_api.model.x509_root_certificate_spec import X509RootCertificateSpec
from agilicus_api.model.x509_root_certificate_status import X509RootCertificateStatus
from agilicus_api.model.xss_settings import XSSSettings





import csv
import sys
import agilicus
import click
import dateparser
import datetime

from typing import Any, Dict

from . import context


def update_if_not_none(obj: dict, new_values: dict):
    for k, v in new_values.items():
        if v is not None and (not isinstance(v, tuple) or len(v)):
            obj[k] = v


def strip_none(obj: dict):
    to_return = {}
    update_if_not_none(to_return, obj)
    return to_return


def update_if_present(object: dict, key, **kwargs):
    value = kwargs.get(key, None)
    if value is not None:
        object[key] = value


def pop_item_if_none(obj: dict, key=None):
    if not obj:
        return

    keys_to_pop = []
    if key is not None:
        if obj.get(key, True) is None:
            keys_to_pop.append(key)
    else:
        for k, v in obj.items():
            if v is None:
                keys_to_pop.append(k)
    for k in keys_to_pop:
        obj.pop(k)
    return obj


def build_updated_model(klass, model, new_values, check_type=True):
    model_dict = model.to_dict()
    update_if_not_none(model_dict, new_values)
    return klass._from_openapi_data(**model_dict, _check_type=check_type)


def model_from_dict(klass, model_dict, check_type=True):
    return agilicus.agilicus_api.api_client.validate_and_convert_types(
        input_value=model_dict,
        required_types_mixed=[klass],
        path_to_item=[""],
        spec_property_naming=True,
        _check_type=check_type,
        configuration=agilicus.agilicus_api.api_client.Configuration(),
    )


def build_updated_model_validate(klass, model, new_values, check_type=True):
    """
    Like build_updated_model, but uses validate_and_convert_types which handles
    subtypes
    """
    model_dict = model.to_dict()
    update_if_not_none(model_dict, new_values)
    return model_from_dict(klass, model_dict, check_type)


def update_attrs_if_not_none(model, new_values):
    for k, v in new_values.items():
        if hasattr(model, k) and v is not None and v != tuple():
            setattr(model, k, v)
    return model


def update_org_from_input_or_ctx(params, *args, **kwargs):
    org_id = get_org_from_input_or_ctx(*args, **kwargs)
    if org_id is not None:
        params["org_id"] = org_id
    else:
        params.pop("org_id", None)


def get_org_from_input_or_ctx(ctx, org_id=None, **kwargs):
    if org_id is None:
        token = context.get_token(ctx)
        org_id = context.get_org_id(ctx, token)

    # Treat an empty-string org id like None so that we can query all if necessary
    if org_id == "":
        org_id = None

    return org_id


def get_user_id_from_input_or_ctx(ctx, user_id=None, **kwargs):
    if user_id is None:
        token = context.get_token(ctx)
        user_id = context.get_user_id(ctx, token)

    # Treat an empty-string like None so that we can query all if necessary
    if user_id == "":
        user_id = None

    return user_id


def parse_csv_input(input_filename, parser):
    input_file = sys.stdin
    if input_filename != "-":
        input_file = open(input_filename, "r")

    results = list()
    with input_file:
        csv_input = csv.DictReader(input_file, delimiter=",", quotechar='"')

        for result_dict in csv_input:
            result = parser(result_dict)
            if result:
                results.append(result)

    return results


class SubObjectType(click.ParamType):
    """
    SubObjectType allows us to map an input value to a specific location in an object
    we're building up.
      `location` identifies the sub object to map to (e.g. metadata).
      `base_type` identifies the type we're wrapping (e.g. click.INT)
    use `get_object_by_location` to extract the k,v pairs for a specific location.
    """

    def __init__(self, location, base_type: click.ParamType):
        self.location = location
        self.base_type = base_type
        self.name = base_type.name
        self._value = None

    def convert(self, value, param, ctx):
        self._value = self.base_type(value, param, ctx)
        return self

    def value(self):
        return self._value


def build_alternate_mode_setting(
    existing_setting, learning_mode, learning_mode_expiry, diagnostic_mode: bool
):
    if existing_setting is None:
        existing_setting = agilicus.AlternateModeSetting()

    if learning_mode is False:
        if existing_setting.learning_mode:
            del existing_setting["learning_mode"]
    elif learning_mode is not None or learning_mode_expiry is not None:
        existing_setting.learning_mode = agilicus.LearningModeSpec()
        if learning_mode_expiry is not None:
            existing_setting.learning_mode.expiry_time = learning_mode_expiry

    if diagnostic_mode is not None:
        existing_setting.diagnostic_mode = diagnostic_mode

    return existing_setting


class SubObjectString(SubObjectType):
    def __init__(self, location):
        super().__init__(location, click.STRING)


class SubObjectInt(SubObjectType):
    def __init__(self, location):
        super().__init__(location, click.INT)


def get_objects_by_location(location, objects: Dict[str, Any]) -> Dict[str, Any]:
    result = {}
    for key, obj in objects.items():
        if not isinstance(obj, SubObjectType):
            continue
        if obj.location != location:
            continue

        val = obj.value()
        if val is None:
            continue

        result[key] = val

    return result


class HumanReadableDateType(click.ParamType):
    """
    HumanReadableDateType allows us accept human-readable date, converting them
    into proper datetime objects.
    """

    settings = {"RETURN_AS_TIMEZONE_AWARE": True}
    name = "human readable datetime"

    def __init__(self):
        self._value = None

    def convert(self, value, param, ctx):
        if isinstance(value, datetime.datetime):
            return value
        try:
            result = dateparser.parse(value, settings=self.settings)
        except Exception as exc:
            print(f"failed to parse: {exc}")
            self.fail(str(exc), param, ctx)
        if result is None:
            self.fail(
                (
                    f"'{value}' is not a valid human-readable date. "
                    "E.g. '2024-05-01T00:20:20.000Z' or 'yesterday'"
                ),
                param=param,
                ctx=ctx,
            )

        return result


def add_remove_uniq_list(input_list, to_add_list, to_remove_list):
    if input_list is None:
        input_list = []
    item_set = set(input_list)

    for item in to_add_list or []:
        item_set.add(item)

    for item in to_remove_list or []:
        item_set.remove(item)
    return list(item_set)


class EnumType(click.Choice):
    """
    A wrapper around a python enum, inspired by
    https://github.com/pallets/click/issues/605
    """

    def __init__(self, enum):
        self.__enum = enum
        super().__init__([self._to_ui(name) for name in enum.__members__])

    @classmethod
    def _to_ui(cls, enum_name: str):
        return enum_name.replace("_", "-")

    @classmethod
    def _from_ui(cls, enum_name: str):
        return enum_name.replace("-", "_")

    def convert(self, value, param, ctx):
        return self.__enum[self._from_ui(super().convert(value, param, ctx))]



from agilicus.input_helpers import pop_item_if_none, strip_none
from . import context
from . import connectors
from . import orgs
from .output.table import (
    column,
    format_table,
)
import operator


def query_top(
    ctx,
    org_id=None,
    dt_from=None,
    dt_to=None,
    app_id=None,
    sub_org_id=None,
    interval=None,
    limit=None,
    **kwargs,
):
    token = context.get_token(ctx)

    if not org_id:
        org_id = context.get_org_id(ctx, token)

    apiclient = context.get_apiclient(ctx, token)

    pop_item_if_none(kwargs)
    params = strip_none(kwargs)
    params["dt_from"] = dt_from
    params["dt_to"] = dt_to

    if app_id:
        params["app_id"] = app_id
    if interval:
        params["interval"] = int(interval)
    if limit:
        params["limit"] = int(limit)

    resp = apiclient.metrics_api.list_top_users(org_id, **params)

    return resp.top_users


def query_active(
    ctx,
    org_id=None,
    dt_from=None,
    dt_to=None,
    app_id=None,
    sub_org_id=None,
    interval=None,
    **kwargs,
):
    token = context.get_token(ctx)

    if not org_id:
        org_id = context.get_org_id(ctx, token)

    apiclient = context.get_apiclient(ctx, token)

    params = strip_none(kwargs)
    pop_item_if_none(kwargs)
    params["dt_from"] = dt_from
    params["dt_to"] = dt_to

    if app_id:
        params["app_id"] = app_id
    if interval:
        params["interval"] = int(interval)

    resp = apiclient.metrics_api.list_active_users(org_id, **params)

    return resp.active_users


def show_top_connectors(ctx, n=10, org_id=None):
    by_org = {}
    for connector in connectors.query(ctx, org_id=org_id):
        conns = by_org.setdefault(connector.spec.org_id, [])
        conns.append(connector)

    org_by_id, _ = orgs.get_org_by_dictionary(ctx, org_id)
    rows = []
    for k, v in sorted(by_org.items(), key=lambda item: len(item[1]), reverse=True):
        if len(rows) >= n:
            break
        org_name = org_by_id.get(k, {}).get("organisation", "unknown org")
        rows.append({"org_id": k, "Organisation": org_name, "num_connectors": len(v)})
    columns = [
        column("org_id"),
        column("Organisation"),
        column("num_connectors"),
    ]
    print(format_table(ctx, rows, columns, getter=operator.itemgetter))



from . import context
from .input_helpers import get_org_from_input_or_ctx, strip_none
from .input_helpers import pop_item_if_none
import agilicus
from .output.table import (
    spec_column,
    format_table,
    metadata_column,
    status_column,
    subtable,
)

TARGET_ISSUER_VALUES = [
    "agilicus-private",
    "agilicus-public-acme",
]


ReasonEnum = agilicus.CSRReasonEnum.allowed_values[("value",)]


def add_agent_csr(ctx, connector_id, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs.pop("org_id", None)
    kwargs = strip_none(kwargs)

    if kwargs["request"] is not None:
        csr_request = open(kwargs["request"], "r").read()
        kwargs["request"] = csr_request

    spec = agilicus.CertSigningReqSpec(org_id=org_id, **kwargs)
    req = agilicus.CertSigningReq(spec=spec)
    return apiclient.connectors_api.create_agent_csr(connector_id, req)


def list_agent_csr(ctx, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs.pop("org_id", None)
    kwargs = strip_none(kwargs)

    query_results = apiclient.connectors_api.list_agent_csr(
        org_id=org_id, **kwargs
    ).certificate_signing_requests
    return query_results


def get_agent_csr(ctx, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs.pop("org_id", None)

    return apiclient.connectors_api.get_agent_csr(
        org_id=org_id,
        **kwargs,
    )


def reissue_csr(ctx, csr_id, org_id, old_not_after, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs.pop("org_id", None)

    spec = agilicus.CertSigningReqReissueSpec(
        csr_id=csr_id, org_id=org_id, old_not_after=old_not_after
    )

    req = agilicus.CertSigningReqReissue(
        spec=spec,
    )

    return apiclient.certificates_api.reissue_cert_for_csr(
        cert_signing_req_reissue=req,
    )


def list_csr(ctx, reason=None, not_valid_after=None, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs.pop("org_id", None)
    if reason is not None:
        kwargs["reason"] = reason

    if org_id:
        kwargs["org_id"] = org_id

    if not_valid_after is not None:
        kwargs["not_valid_after"] = not_valid_after

    kwargs = strip_none(kwargs)
    query_results = apiclient.certificates_api.list_csr(
        **kwargs
    ).certificate_signing_requests
    return query_results


def get_csr(ctx, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    pop_item_if_none(kwargs)
    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs.pop("org_id", None)

    return apiclient.certificates_api.get_csr(org_id=org_id, **kwargs)


def delete_csr(ctx, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs.pop("org_id", None)

    return apiclient.certificates_api.delete_csr(org_id=org_id, **kwargs)


def format_csr_as_text(ctx, csrs, get_certificate_updates=False, **kwargs):
    certificate_columns = [
        spec_column("reason"),
        status_column("not_after"),
    ]
    columns = [
        metadata_column("id"),
        spec_column("org_id"),
        spec_column("private_key_id", "key_id", max_size=8),
        spec_column("target_issuer"),
        status_column("connector_id"),
        status_column("auto_renew"),
        status_column("dns_names"),
        subtable(ctx, "certificates", certificate_columns, subobject_name="status"),
    ]

    if get_certificate_updates:
        certificate_updates_columns = [
            metadata_column("created"),
            spec_column("message"),
        ]
        columns.append(
            subtable(
                ctx,
                "certificate_updates",
                certificate_updates_columns,
                subobject_name="status",
            )
        )

    return format_table(ctx, csrs, columns)


def add_csr(ctx, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs.pop("org_id", None)
    kwargs = strip_none(kwargs)

    if kwargs["request"] is not None:
        csr_request = open(kwargs["request"], "r").read()
        kwargs["request"] = csr_request

    spec = agilicus.CertSigningReqSpec(org_id=org_id, **kwargs)
    req = agilicus.CertSigningReq(spec=spec)
    return apiclient.connectors_api.create_csr(req)



import datetime

import agilicus
import shortuuid

from dataclasses import dataclass
from enum import Enum
from typing import Optional


from agilicus_api import (
    Message,
    MessageAction,
    MessageAddress,
    MessageClass,
    MessageType,
    MessageSendRequest,
    MessageSendItem,
    MessageTagName,
    MessageTag,
    MessagesBulkDeleteRequest,
)

from .. import context
from .. import input_helpers
from ..output.json import convert_to_json
from ..output.table import (
    format_table,
    metadata_column,
    spec_column,
    status_column,
)


def send_broadcast_from_cli(
    ctx,
    user_org_id,
    expiry,
    title,
    text,
    icon_url,
    link,
    target,
    message_type,
    campaign_id,
    push_probability,
):
    if not is_tz_aware(expiry):
        expiry = expiry.astimezone(datetime.timezone.utc)
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    if not campaign_id:
        campaign_id = shortuuid.uuid()
    msg = Broadcast(
        title=title,
        text=text,
        message_type=message_type,
        icon_url=icon_url,
        link=link,
        push_probability=push_probability,
    )

    return send_broadcast(
        apiclient.messages_api,
        user_org_id_list=user_org_id,
        msg=msg,
        expiry=expiry,
        target=target,
        campaign_id=campaign_id,
    )


class BroadcastType(Enum):
    message_of_the_day = "motd"
    survey = "survey"


class MessageTarget(Enum):
    admin_portal = "agilicus-admin-portal"
    profile = "agilicus-profile"


@dataclass
class Broadcast:
    title: str
    text: str
    message_type: BroadcastType
    icon_url: Optional[str] = None
    link: Optional[str] = None
    push_probability: Optional[float] = None


def send_broadcast(
    api: agilicus.MessagesApi,
    user_org_id_list: list[tuple[str, str] | tuple[str, str, bool]],
    msg: Broadcast,
    expiry: datetime.datetime,
    target: MessageTarget,
    campaign_id: str,
):
    """
    `send_broadcast` sends a message to a list of users (in a given org), which will
    expire (be removed from their inbox automatically). This can be used to
    broadcast important announcements, solicit feedback, notify of new features,
    etc.

    Each user is identified in the `user_org_id_list`, the first item being the user
    id, and the second their org id. If you want to send to a group of users, use the
    group's user id as the first parameter in the list, and set the third, optional,
    parameter to `True`.
    """
    if not is_tz_aware(expiry):
        raise ValueError(f"expiry ({expiry}) must be timezone aware")

    message = agilicus.Message(
        text=msg.text,
        title=msg.title,
        message_type=agilicus.MessageType(msg.message_type.value),
        message_class=agilicus.MessageClass(target.value),
    )

    if msg.icon_url is not None:
        message.icon_url = msg.icon_url
    if msg.link is not None:
        message.uri = msg.link
    if msg.push_probability is not None:
        message.push_probability = msg.push_probability

    message.expiry_date = expiry
    message.tag = agilicus.MessageTag(tag_name=agilicus.MessageTagName(campaign_id))

    send_item = agilicus.MessageSendItem(
        message=message,
        ephemeral=False,
        addresses=[
            _user_org_tuple_to_address(user_org) for user_org in user_org_id_list
        ],
        endpoint_types=[agilicus.MessageEndpointType("inbox")],
    )
    req = agilicus.MessageSendRequest(
        messages=[send_item],
    )
    return api.create_routed_message(req)


def _user_org_tuple_to_address(user_org: tuple[str, str] | tuple[str, str, bool]):
    addr = agilicus.MessageAddress(
        user_id=user_org[0],
        org_id=user_org[1],
        direct=True,
    )

    if len(user_org) > 2:
        # The third parameter here stands for "send to group", which is the inverse of
        # the direct field of the MessageAddress.
        addr.direct = not user_org[2]

    return addr


def is_tz_aware(dt: datetime.datetime):
    return dt.tzinfo is not None and dt.tzinfo.utcoffset(dt) is not None


def list_message_endpoints(ctx, user_id, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    if user_id:
        kwargs["user_id"] = user_id
    results = apiclient.messages_api.list_message_endpoints(**kwargs)
    return results.messages


def format_message_endpoints(ctx, details):
    columns = [
        metadata_column("message_endpoint_id"),
        metadata_column("user_id", "User ID"),
        spec_column("endpoint_type", "Type"),
        spec_column("nickname", "Nickname"),
        spec_column("address", "Address"),
    ]

    return format_table(ctx, details, columns)


def delete_message_endpoint(ctx, message_endpoint_id, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    user_id = input_helpers.get_user_id_from_input_or_ctx(ctx, **kwargs)
    params = {}
    if user_id:
        params["user_id"] = user_id
    resp = apiclient.messages_api.delete_message_endpoint(message_endpoint_id, **params)
    return resp


def get_message_endpoint(ctx, message_endpoint_id, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    result = apiclient.messages_api.get_message_endpoint(message_endpoint_id, **kwargs)
    return result


def update_message_endpoint(ctx, user_id, message_endpoint_id, enabled=None, **kwargs):
    user_id = input_helpers.get_user_id_from_input_or_ctx(ctx, user_id, **kwargs)
    apiclient = context.get_apiclient_from_ctx(ctx)
    endpoint = apiclient.messages_api.get_message_endpoint(
        user_id=user_id, message_endpoint_id=message_endpoint_id
    )

    if enabled is not None:
        endpoint.spec.enabled = enabled

    return apiclient.messages_api.replace_message_endpoint(message_endpoint_id, endpoint)


def send_message(
    ctx,
    user_id,
    text,
    org_id=None,
    message_class=None,
    message_type=None,
    sub_header=None,
    approve_uri=None,
    reject_uri=None,
    tag=None,
    **kwargs,
):
    user_id = input_helpers.get_user_id_from_input_or_ctx(ctx, user_id, **kwargs)
    kwargs = input_helpers.strip_none(kwargs)
    message = Message(text=text, **kwargs)
    if message_class is not None:
        message.message_class = MessageClass(message_class)
    if message_type is not None:
        message.message_type = MessageType(message_type)

    message.actions = []
    if approve_uri is not None:
        message.actions.append(MessageAction(title="approve", uri=approve_uri))

    if reject_uri is not None:
        message.actions.append(MessageAction(title="reject", uri=reject_uri))

    if tag is not None:
        message.tag = make_message_tag(tag, org_id)

    address = MessageAddress(user_id=user_id)
    if org_id is not None:
        address.org_id = org_id
    else:
        address.org_id = input_helpers.get_org_from_input_or_ctx(ctx, **kwargs)
    item = MessageSendItem(message=message, addresses=[address], ephemeral=False)
    req = MessageSendRequest(messages=[item])

    apiclient = context.get_apiclient_from_ctx(ctx)
    return apiclient.messages_api.create_routed_message(req)


def make_message_tag(tag, org_id=None):
    if tag is None:
        return None
    result = MessageTag(MessageTagName(tag))
    if org_id is not None:
        result.org_id = org_id
    return result


def list_inbox_items(ctx, user_id, **kwargs):
    user_id = input_helpers.get_user_id_from_input_or_ctx(ctx, user_id, **kwargs)
    apiclient = context.get_apiclient_from_ctx(ctx)
    kwargs = input_helpers.strip_none(kwargs)
    return apiclient.messages_api.list_inbox_items(user_id=user_id, **kwargs)


def update_inbox_item(ctx, user_id, inbox_item_id, has_been_read=None, **kwargs):
    user_id = input_helpers.get_user_id_from_input_or_ctx(ctx, user_id, **kwargs)
    apiclient = context.get_apiclient_from_ctx(ctx)
    item = apiclient.messages_api.get_inbox_item(
        user_id=user_id, inbox_item_id=inbox_item_id
    )
    if has_been_read is not None:
        item.spec.has_been_read = has_been_read
    return apiclient.messages_api.replace_inbox_item(
        inbox_item=item, user_id=user_id, inbox_item_id=inbox_item_id
    )


def bulk_delete_messages(ctx, tag, **kwargs):
    kwargs = input_helpers.strip_none(kwargs)
    msg_tag = make_message_tag(tag)
    if msg_tag:
        kwargs["message_tag"] = msg_tag
    req = MessagesBulkDeleteRequest(**kwargs)
    apiclient = context.get_apiclient_from_ctx(ctx)
    resp = apiclient.messages_api.bulk_delete_messages(req)
    return resp


def format_inbox_items_response(ctx, response):
    columns = [
        metadata_column("id"),
        metadata_column("created"),
        metadata_column("org_id"),
        spec_column("has_been_read", "read"),
        status_column("message.message_class", "class"),
        status_column("message.message_type", "type"),
        status_column("message.title", "title"),
        status_column("message.expiry_date", "expiry"),
    ]
    if context.output_json(ctx):
        return convert_to_json(ctx, response.to_dict())

    return format_table(ctx, response.inbox_items, columns)



"""
    Agilicus API

    Agilicus is API-first. Modern software is controlled by other software, is open, is available for you to use the way you want, securely, simply.  The OpenAPI Specification in YAML format is available on [www](https://www.agilicus.com/www/api/agilicus-openapi.yaml) for importing to other tools.  A rendered, online viewable and usable version of this specification is available at [api](https://www.agilicus.com/api). You may try the API inline directly in the web page. To do so, first obtain an Authentication Token (the simplest way is to install the Python SDK, and then run `agilicus-cli --issuer https://MYISSUER get-token`). You will need an org-id for most calls (and can obtain from `agilicus-cli --issuer https://MYISSUER list-orgs`). The `MYISSUER` will typically be `auth.MYDOMAIN`, and you will see it as you sign-in to the administrative UI.  This API releases on Bearer-Token authentication. To obtain a valid bearer token you will need to Authenticate to an Issuer with OpenID Connect (a superset of OAUTH2).  Your \"issuer\" will look like https://auth.MYDOMAIN. For example, when you signed-up, if you said \"use my own domain name\" and assigned a CNAME of cloud.example.com, then your issuer would be https://auth.cloud.example.com.  If you selected \"use an Agilicus supplied domain name\", your issuer would look like https://auth.myorg.agilicus.cloud.  For test purposes you can use our [Python SDK](https://pypi.org/project/agilicus/) and run `agilicus-cli --issuer https://auth.MYDOMAIN get-token`.  This API may be used in any language runtime that supports OpenAPI 3.0, or, you may use our [Python SDK](https://pypi.org/project/agilicus/), our [Typescript SDK](https://www.npmjs.com/package/@agilicus/angular), or our [Golang SDK](https://git.agilicus.com/pub/sdk-go).  100% of the activities in our system our API-driven, from our web-admin, through our progressive web applications, to all internals: there is nothing that is not accessible.  For more information, see [developer resources](https://www.agilicus.com/developer).   # noqa: E501

    The version of the OpenAPI document: 2025.02.12
    Contact: dev@agilicus.com
    Generated by: https://openapi-generator.tech
"""


import unittest

import agilicus_api
from agilicus_api.api.users_api import UsersApi  # noqa: E501


class TestUsersApi(unittest.TestCase):
    """UsersApi unit test stubs"""

    def setUp(self):
        self.api = UsersApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_bulk_approve_requests(self):
        """Test case for bulk_approve_requests

        Approve a list of requests on behalf of users  # noqa: E501
        """
        pass

    def test_bulk_update_metadata(self):
        """Test case for bulk_update_metadata

        Update a group of user's metadata for the specified org  # noqa: E501
        """
        pass

    def test_create_challenge_method(self):
        """Test case for create_challenge_method

        Create a multi-factor authentication method  # noqa: E501
        """
        pass

    def test_create_service_account(self):
        """Test case for create_service_account

        Create a service account  # noqa: E501
        """
        pass

    def test_create_suppport_user(self):
        """Test case for create_suppport_user

        Create a support user  # noqa: E501
        """
        pass

    def test_create_upstream_user_identity(self):
        """Test case for create_upstream_user_identity

        Create an upstream user identity  # noqa: E501
        """
        pass

    def test_create_user(self):
        """Test case for create_user

        Create a user  # noqa: E501
        """
        pass

    def test_create_user_identity_update(self):
        """Test case for create_user_identity_update

        Update a user's core identity information.  # noqa: E501
        """
        pass

    def test_create_user_metadata(self):
        """Test case for create_user_metadata

        Create a metadata entry for the user  # noqa: E501
        """
        pass

    def test_create_user_request(self):
        """Test case for create_user_request

        Create a request on behalf of the user  # noqa: E501
        """
        pass

    def test_delete_challenge_method(self):
        """Test case for delete_challenge_method

        Delete a user's multi-factor authentication challenge method  # noqa: E501
        """
        pass

    def test_delete_service_account(self):
        """Test case for delete_service_account

        Delete a service account  # noqa: E501
        """
        pass

    def test_delete_support_user(self):
        """Test case for delete_support_user

        Delete a support user  # noqa: E501
        """
        pass

    def test_delete_upstream_user_identity(self):
        """Test case for delete_upstream_user_identity

        Delete an upstream user identity  # noqa: E501
        """
        pass

    def test_delete_user(self):
        """Test case for delete_user

        Remove a user from an organisation  # noqa: E501
        """
        pass

    def test_delete_user_metadata(self):
        """Test case for delete_user_metadata

        Delete an user metadata entry  # noqa: E501
        """
        pass

    def test_delete_user_request(self):
        """Test case for delete_user_request

        Delete an user request  # noqa: E501
        """
        pass

    def test_get_challenge_method(self):
        """Test case for get_challenge_method

        Get a single challenge method for the given user  # noqa: E501
        """
        pass

    def test_get_service_account(self):
        """Test case for get_service_account

        Get a service account  # noqa: E501
        """
        pass

    def test_get_support_user(self):
        """Test case for get_support_user

        Get a support user  # noqa: E501
        """
        pass

    def test_get_upstream_user_identity(self):
        """Test case for get_upstream_user_identity

        Get a single upstream user identity  # noqa: E501
        """
        pass

    def test_get_user(self):
        """Test case for get_user

        Get a single user  # noqa: E501
        """
        pass

    def test_get_user_metadata(self):
        """Test case for get_user_metadata

        Get a single user metadata entry  # noqa: E501
        """
        pass

    def test_get_user_request(self):
        """Test case for get_user_request

        Get a single user request  # noqa: E501
        """
        pass

    def test_list_access_requests(self):
        """Test case for list_access_requests

        Get a list of access requests  # noqa: E501
        """
        pass

    def test_list_all_resource_permissions(self):
        """Test case for list_all_resource_permissions

        Return all per-resource permissions for a user  # noqa: E501
        """
        pass

    def test_list_all_user_orgs(self):
        """Test case for list_all_user_orgs

        Return all organisations a user has been assigned to  # noqa: E501
        """
        pass

    def test_list_all_user_roles(self):
        """Test case for list_all_user_roles

        Return all roles for a user  # noqa: E501
        """
        pass

    def test_list_challenge_methods(self):
        """Test case for list_challenge_methods

        Get all of a user's multi-factor authentication challenge method configuration  # noqa: E501
        """
        pass

    def test_list_combined_user_details(self):
        """Test case for list_combined_user_details

        Get all combined details about users  # noqa: E501
        """
        pass

    def test_list_desktop_access_info(self):
        """Test case for list_desktop_access_info

        Query various users' desktop access information  # noqa: E501
        """
        pass

    def test_list_org_user_roles(self):
        """Test case for list_org_user_roles

        Get all org user roles  # noqa: E501
        """
        pass

    def test_list_service_accounts(self):
        """Test case for list_service_accounts

        List service accounts  # noqa: E501
        """
        pass

    def test_list_ssh_access_info(self):
        """Test case for list_ssh_access_info

        Query various users' SSH access information  # noqa: E501
        """
        pass

    def test_list_support_users(self):
        """Test case for list_support_users

        List support users  # noqa: E501
        """
        pass

    def test_list_upstream_user_identities(self):
        """Test case for list_upstream_user_identities

        Get all of a user's upstream user identities  # noqa: E501
        """
        pass

    def test_list_user_application_access_info(self):
        """Test case for list_user_application_access_info

        Query various users' application access information  # noqa: E501
        """
        pass

    def test_list_user_file_share_access_info(self):
        """Test case for list_user_file_share_access_info

        Query various users' file share access information  # noqa: E501
        """
        pass

    def test_list_user_guid_mapping(self):
        """Test case for list_user_guid_mapping

        Get all user guids and a unique name mapping  # noqa: E501
        """
        pass

    def test_list_user_guids(self):
        """Test case for list_user_guids

        Get a list of all user GUIDs  # noqa: E501
        """
        pass

    def test_list_user_launcher_access_info(self):
        """Test case for list_user_launcher_access_info

        Query various users' launchers access information  # noqa: E501
        """
        pass

    def test_list_user_metadata(self):
        """Test case for list_user_metadata

        Get a list of user metadata entries  # noqa: E501
        """
        pass

    def test_list_user_permissions(self):
        """Test case for list_user_permissions

        Return the user's host permissions  # noqa: E501
        """
        pass

    def test_list_user_requests(self):
        """Test case for list_user_requests

        Get a list of user requests  # noqa: E501
        """
        pass

    def test_list_user_resource_access_info(self):
        """Test case for list_user_resource_access_info

        Query various users' resource access information  # noqa: E501
        """
        pass

    def test_list_users(self):
        """Test case for list_users

        Get all users  # noqa: E501
        """
        pass

    def test_replace_challenge_method(self):
        """Test case for replace_challenge_method

        Update a user's multi-factor authentication challenge method  # noqa: E501
        """
        pass

    def test_replace_service_account(self):
        """Test case for replace_service_account

        Update a service account  # noqa: E501
        """
        pass

    def test_replace_upstream_user_identity(self):
        """Test case for replace_upstream_user_identity

        Update an upstream user identity  # noqa: E501
        """
        pass

    def test_replace_user(self):
        """Test case for replace_user

        Create or update a user  # noqa: E501
        """
        pass

    def test_replace_user_metadata(self):
        """Test case for replace_user_metadata

        Update an user metadata entry.  # noqa: E501
        """
        pass

    def test_replace_user_request(self):
        """Test case for replace_user_request

        Update an user request. Note this method ignores the state parameter.  # noqa: E501
        """
        pass

    def test_replace_user_role(self):
        """Test case for replace_user_role

        Create or update a user role  # noqa: E501
        """
        pass

    def test_reset_user_identity(self):
        """Test case for reset_user_identity

        Resets a user's identity if allowed  # noqa: E501
        """
        pass

    def test_reset_user_mfa_challenge_methods(self):
        """Test case for reset_user_mfa_challenge_methods

        Resets a user's multi-factor authentication method  # noqa: E501
        """
        pass

    def test_update_support_user(self):
        """Test case for update_support_user

        Update a support user's expiry  # noqa: E501
        """
        pass

    def test_update_user_request(self):
        """Test case for update_user_request

        Uses the state parameter in the body to apply the action to the request  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()
